@@ -6,7 +6,9 @@
 #include <glibtop/close.h>
 #include <glibtop/cpu.h>
 #include <glibtop/sysinfo.h>
+#include <libhandy-1/handy.h>
 #include <signal.h>
+#include <stdlib.h>
 
 #include "application.h"
 #include "procdialogs.h"
@@ -30,6 +32,23 @@ cb_solaris_mode_changed (Gio::Settings& settings, Glib::ustring key, GsmApplicat
     }
 }
 
+static void
+cb_process_memory_in_iec_changed (Gio::Settings& settings, Glib::ustring key, GsmApplication* app) {
+    app->config.process_memory_in_iec = settings.get_boolean(key);
+    app->cpu_graph->clear_background();
+    if (app->timeout) {
+        proctable_update(app);
+    }
+}
+
+static void
+cb_logarithmic_scale_changed (Gio::Settings& settings, Glib::ustring key, GsmApplication* app)
+{
+    app->config.logarithmic_scale = settings.get_boolean(key);
+    app->mem_graph->clear_background();
+    load_graph_reset(app->mem_graph);
+}
+
 static void
 cb_draw_stacked_changed (Gio::Settings& settings, Glib::ustring key, GsmApplication* app)
 {
@@ -43,13 +62,51 @@ cb_draw_smooth_changed (Gio::Settings& settings, Glib::ustring key, GsmApplicati
 {
     app->config.draw_smooth = settings.get_boolean(key);
     app->cpu_graph->clear_background();
+    app->mem_graph->clear_background();
+    app->net_graph->clear_background();
     load_graph_reset(app->cpu_graph);
+    load_graph_reset(app->mem_graph);
+    load_graph_reset(app->net_graph);
+}
+
+static void
+cb_resources_memory_in_iec_changed (Gio::Settings& settings, Glib::ustring key, GsmApplication* app)
+{
+    app->config.resources_memory_in_iec = settings.get_boolean(key);
+    app->cpu_graph->clear_background();
+    if (app->timeout) {
+        proctable_update (app);
+    }
 }
 
 static void
 cb_network_in_bits_changed (Gio::Settings& settings, Glib::ustring key, GsmApplication* app)
 {
     app->config.network_in_bits = settings.get_boolean(key);
+    if (app->config.network_total_unit == FALSE) {
+        app->config.network_total_in_bits = app->config.network_in_bits;
+    }
+    // force scale to be redrawn
+    app->net_graph->clear_background();
+}
+
+static void
+cb_network_total_in_unit_changed (Gio::Settings& settings, Glib::ustring key, GsmApplication* app)
+{
+    app->config.network_total_unit = settings.get_boolean(key);
+    if (app->config.network_total_unit == FALSE) {
+        app->config.network_total_in_bits = app->config.network_in_bits;
+    } else {
+        app->config.network_total_in_bits = app->settings->get_boolean (GSM_SETTING_NETWORK_TOTAL_IN_BITS);
+    }
+    // force scale to be redrawn
+    app->net_graph->clear_background();
+}
+
+static void
+cb_network_total_in_bits_changed (Gio::Settings& settings, Glib::ustring key, GsmApplication* app)
+{
+    app->config.network_total_in_bits = settings.get_boolean(key);
     // force scale to be redrawn
     app->net_graph->clear_background();
 }
@@ -78,6 +135,15 @@ cb_timeouts_changed (Gio::Settings& settings, Glib::ustring key, GsmApplication*
     }
 }
 
+static void
+cb_data_points_changed(Gio::Settings& settings, Glib::ustring key, GsmApplication* app) {
+    app->config.graph_data_points = settings.get_int (key);
+    unsigned points = app->config.graph_data_points + 2;
+    load_graph_change_num_points(app->cpu_graph, points);
+    load_graph_change_num_points(app->mem_graph, points);
+    load_graph_change_num_points(app->net_graph, points);
+}
+
 static void
 apply_cpu_color_settings(Gio::Settings& settings, GsmApplication* app)
 {
@@ -93,6 +159,8 @@ apply_cpu_color_settings(Gio::Settings& settings, GsmApplication* app)
 
     g_variant_builder_init(&builder, G_VARIANT_TYPE_ARRAY);
 
+    std::vector<std::string> random_colors = procman::generate_colors(app->config.num_cpus);
+
     for (guint i = 0; i < static_cast<guint>(app->config.num_cpus); i++) {
         if(i < n) {
             child = g_variant_get_child_value ( cpu_colors_var, i );
@@ -100,7 +168,7 @@ apply_cpu_color_settings(Gio::Settings& settings, GsmApplication* app)
             g_variant_builder_add_value ( &builder, child);
             g_variant_unref (child);
         } else {
-            color = g_strdup ("#f25915e815e8");
+            color = g_strdup (random_colors[i].c_str());
             g_variant_builder_add(&builder, "(us)", i, color);
         }
         gdk_rgba_parse(&app->config.cpu_color[i], color);
@@ -159,6 +227,16 @@ GsmApplication::load_settings()
         cb_solaris_mode_changed (*this->settings.operator->(), key, this);
     });
 
+    config.process_memory_in_iec = this->settings->get_boolean (GSM_SETTING_PROCESS_MEMORY_IN_IEC);
+    this->settings->signal_changed (GSM_SETTING_PROCESS_MEMORY_IN_IEC).connect ([this](const Glib::ustring& key) {
+        cb_process_memory_in_iec_changed (*this->settings.operator->(), key, this);
+	});
+
+    config.logarithmic_scale = this->settings->get_boolean (GSM_SETTING_LOGARITHMIC_SCALE);
+    this->settings->signal_changed(GSM_SETTING_LOGARITHMIC_SCALE).connect ([this](const Glib::ustring& key) {
+        cb_logarithmic_scale_changed (*this->settings.operator->(), key, this);
+    });
+
     config.draw_stacked = this->settings->get_boolean (GSM_SETTING_DRAW_STACKED);
     this->settings->signal_changed(GSM_SETTING_DRAW_STACKED).connect ([this](const Glib::ustring& key) {
         cb_draw_stacked_changed (*this->settings.operator->(), key, this);
@@ -169,11 +247,31 @@ GsmApplication::load_settings()
         cb_draw_smooth_changed (*this->settings.operator->(), key, this);
     });
 
+    config.resources_memory_in_iec = this->settings->get_boolean (GSM_SETTING_RESOURCES_MEMORY_IN_IEC);
+    this->settings->signal_changed (GSM_SETTING_RESOURCES_MEMORY_IN_IEC).connect ([this](const Glib::ustring& key) {
+        cb_resources_memory_in_iec_changed (*this->settings.operator->(), key, this);
+    });
+
     config.network_in_bits = this->settings->get_boolean (GSM_SETTING_NETWORK_IN_BITS);
     this->settings->signal_changed (GSM_SETTING_NETWORK_IN_BITS).connect ([this](const Glib::ustring& key) {
         cb_network_in_bits_changed (*this->settings.operator->(), key, this);
     });
 
+    config.network_total_unit = this->settings->get_boolean (GSM_SETTING_NETWORK_TOTAL_UNIT);
+    this->settings->signal_changed (GSM_SETTING_NETWORK_TOTAL_UNIT).connect ([this](const Glib::ustring& key) {
+        cb_network_total_in_unit_changed (*this->settings.operator->(), key, this);
+    });
+
+    if (config.network_total_unit == FALSE) {
+        config.network_total_in_bits = config.network_in_bits;
+        this->settings->set_boolean (GSM_SETTING_NETWORK_TOTAL_IN_BITS, config.network_in_bits);
+    } else {
+        config.network_total_in_bits = this->settings->get_boolean (GSM_SETTING_NETWORK_TOTAL_IN_BITS);
+    }
+    this->settings->signal_changed (GSM_SETTING_NETWORK_TOTAL_IN_BITS).connect ([this](const Glib::ustring& key) {
+        cb_network_total_in_bits_changed (*this->settings.operator->(), key, this);
+    });
+
     auto cbtc = [this](const Glib::ustring& key) { cb_timeouts_changed(*this->settings.operator->(), key, this); };
     config.update_interval = this->settings->get_int (GSM_SETTING_PROCESS_UPDATE_INTERVAL);
     this->settings->signal_changed (GSM_SETTING_PROCESS_UPDATE_INTERVAL).connect (cbtc);
@@ -182,6 +280,11 @@ GsmApplication::load_settings()
     config.disks_update_interval = this->settings->get_int (GSM_SETTING_DISKS_UPDATE_INTERVAL);
     this->settings->signal_changed (GSM_SETTING_DISKS_UPDATE_INTERVAL).connect (cbtc);
 
+    config.graph_data_points = this->settings->get_int (GSM_SETTING_GRAPH_DATA_POINTS);
+    this->settings->signal_changed (GSM_SETTING_GRAPH_DATA_POINTS).connect ([this](const Glib::ustring& key) {
+        cb_data_points_changed (*this->settings.operator->(), key, this);
+    });
+
     glibtop_get_cpu (&cpu);
     frequency = cpu.frequency;
 
@@ -285,8 +388,12 @@ int GsmApplication::on_command_line(const Glib::RefPtr<Gio::ApplicationCommandLi
     Glib::OptionContext context;
     context.set_summary(_("A simple process and system monitor."));
     context.set_ignore_unknown_options(true);
+
+    Glib::OptionGroup gtkgroup(gtk_get_option_group(true));
     procman::OptionGroup option_group;
+
     context.set_main_group(option_group);
+    context.add_group (gtkgroup);
 
     try {
         context.parse(argc, argv);
@@ -365,6 +472,8 @@ void GsmApplication::on_startup()
 {
     Gtk::Application::on_startup();
 
+    hdy_init();
+
     load_resources ();
 
     Glib::RefPtr<Gio::SimpleAction> action;
@@ -396,6 +505,7 @@ void GsmApplication::on_startup()
     add_accelerator("<Primary>c", "win.send-signal-cont", g_variant_new_int32 (SIGCONT));
     add_accelerator("<Primary>e", "win.send-signal-end", g_variant_new_int32 (SIGTERM));
     add_accelerator("<Primary>k", "win.send-signal-kill", g_variant_new_int32 (SIGKILL));
+    add_accelerator("<Alt>s", "win.set-affinity", NULL);
     add_accelerator("<Primary>m", "win.memory-maps", NULL);
     add_accelerator("<Primary>o", "win.open-files", NULL);
     add_accelerator("<Alt>Return", "win.process-properties", NULL);
