@@ -1,6 +1,8 @@
 /* -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 #include <config.h>
 
+#include <math.h>
+
 #include <glib/gi18n.h>
 
 #include <glibtop.h>
@@ -15,6 +17,7 @@
 #include "util.h"
 #include "legacy/gsm_color_button.h"
 
+gchar* format_duration(unsigned seconds);
 
 void LoadGraph::clear_background()
 {
@@ -24,6 +27,11 @@ void LoadGraph::clear_background()
     }
 }
 
+bool LoadGraph::is_logarithmic_scale() const
+{
+    // logarithmic scale is used only for memory graph
+    return this->type == LOAD_GRAPH_MEM && GsmApplication::get()->config.logarithmic_scale;
+}
 
 unsigned LoadGraph::num_bars() const
 {
@@ -43,23 +51,113 @@ unsigned LoadGraph::num_bars() const
         case 4:
             n = 4;
             break;
+        case 5:
+            n = 5;
+            if (this->is_logarithmic_scale())
+                n = 4;
+            break;
         default:
             n = 5;
+            if (this->is_logarithmic_scale())
+                n = 6;
     }
 
     return n;
 }
 
+/*
+ Returns Y scale caption based on give index of the label.
+ Takes into account whether the scale should be logarithmic for memory graph.
+ */
+char* LoadGraph::get_caption(guint index)
+{
+    char *caption;
+    unsigned num_bars = this->num_bars();
+    guint64 max_value;
+    if (this->type == LOAD_GRAPH_NET)
+        max_value = this->net.max;
+    else
+        max_value = 100;
 
+    // operation orders matters so it's 0 if index == num_bars
+    float caption_percentage = (float)max_value - index * (float)max_value / num_bars;
+
+    if (this->is_logarithmic_scale()) {
+        float caption_value = caption_percentage == 0 ? 0 : pow(100, caption_percentage / max_value);
+        caption = g_strdup_printf("%.0f %%", caption_value);
+    } else if (this->type == LOAD_GRAPH_NET) {
+        const std::string captionstr(procman::format_network_rate((guint64)caption_percentage));
+        caption = g_strdup(captionstr.c_str());
+    } else {
+        caption = g_strdup_printf("%.0f %%", caption_percentage);
+    }
+
+    return caption;
+}
+
+/*
+ Translates y partial position to logarithmic position if set to logarithmic scale.
+*/
+float LoadGraph::translate_to_log_partial_if_needed(float position_partial)
+{
+    if (this->is_logarithmic_scale())
+        position_partial = position_partial == 0 ? 0 : log10(position_partial * 100) / 2;
+
+    return position_partial;
+}
+
+gchar* format_duration(unsigned seconds) {
+    gchar* caption = NULL;
+
+    unsigned minutes = seconds / 60;
+    unsigned hours = seconds / 3600;
+
+    if (hours != 0) {
+        if (minutes % 60 == 0) {
+             // If minutes mod 60 is 0 set it to 0, to prevent it from showing full hours in
+             // minutes in addition to hours.
+             minutes = 0;
+        } else {
+             // Round minutes as seconds wont get shown if neither hours nor minutes are 0.
+             minutes = int(rint(seconds / 60.0)) % 60;
+             if (minutes == 0) {
+                  // Increase hours if rounding minutes results in 0, because that would be
+                  // what it would be rounded to.
+                  hours++;
+                  // Set seconds to hours * 3600 to prevent seconds from being drawn.
+                  seconds = hours * 3600;
+             }
+        }
+
+    }
+
+    gchar* captionH = g_strdup_printf(dngettext(GETTEXT_PACKAGE, "%u hr", "%u hrs", hours), hours);
+    gchar* captionM = g_strdup_printf(dngettext(GETTEXT_PACKAGE, "%u min", "%u mins", minutes),
+                                     minutes);
+    gchar* captionS = g_strdup_printf(dngettext(GETTEXT_PACKAGE, "%u sec", "%u secs", seconds % 60),
+                                     seconds % 60);
+
+    caption = g_strjoin (" ", hours > 0 ? captionH : "",
+                              minutes > 0 ? captionM : "",
+                              seconds % 60 > 0 ? captionS : "",
+                         NULL);
+    g_free (captionH);
+    g_free (captionM);
+    g_free (captionS);
+
+    return caption;
+}
 
 const int FRAME_WIDTH = 4;
 static void draw_background(LoadGraph *graph) {
     GtkAllocation allocation;
     cairo_t *cr;
     guint i;
+    double label_x_offset_modifier, label_y_offset_modifier;
     unsigned num_bars;
-    char *caption;
+    gchar *caption;
     PangoLayout* layout;
+    PangoAttrList *attrs = NULL;
     PangoFontDescription* font_desc;
     PangoRectangle extents;
     cairo_surface_t *surface;
@@ -71,8 +169,8 @@ static void draw_background(LoadGraph *graph) {
     num_bars = graph->num_bars();
     graph->graph_dely = (graph->draw_height - 15) / num_bars; /* round to int to avoid AA blur */
     graph->real_draw_height = graph->graph_dely * num_bars;
-    graph->graph_delx = (graph->draw_width - 2.0 - graph->indent) / (LoadGraph::NUM_POINTS - 3);
-    graph->graph_buffer_offset = (int) (1.5 * graph->graph_delx) + FRAME_WIDTH ;
+    graph->graph_delx = (graph->draw_width - 2.0 - graph->indent) / (graph->num_points - 3);
+    graph->graph_buffer_offset = (int) (1.5 * graph->graph_delx) + FRAME_WIDTH;
 
     gtk_widget_get_allocation (GTK_WIDGET (graph->disp), &allocation);
     surface = gdk_window_create_similar_surface (gtk_widget_get_window (GTK_WIDGET (graph->disp)),
@@ -82,12 +180,17 @@ static void draw_background(LoadGraph *graph) {
     cr = cairo_create (surface);
 
     GtkStyleContext *context = gtk_widget_get_style_context (GTK_WIDGET (GsmApplication::get()->stack));
-    
-    gtk_style_context_get_color (context, GTK_STATE_FLAG_NORMAL, &fg);
+
+    gtk_style_context_get_color (context, gtk_widget_get_state_flags (GTK_WIDGET (GsmApplication::get()->stack)), &fg);
 
     cairo_paint_with_alpha (cr, 0.0);
     layout = pango_cairo_create_layout (cr);
-    gtk_style_context_get (context, GTK_STATE_FLAG_NORMAL, GTK_STYLE_PROPERTY_FONT, &font_desc, NULL);
+
+    attrs = make_tnum_attr_list ();
+    pango_layout_set_attributes (layout, attrs);
+    g_clear_pointer (&attrs, pango_attr_list_unref);
+
+    gtk_style_context_get (context, gtk_widget_get_state_flags (GTK_WIDGET (GsmApplication::get()->stack)), GTK_STYLE_PROPERTY_FONT, &font_desc, NULL);
     pango_font_description_set_size (font_desc, 0.8 * graph->fontsize * PANGO_SCALE);
     pango_layout_set_font_description (layout, font_desc);
     pango_font_description_free (font_desc);
@@ -114,7 +217,7 @@ static void draw_background(LoadGraph *graph) {
     gtk_style_context_add_class (context, GTK_STYLE_CLASS_ENTRY);
 
     /* And, as a bonus, the user can choose the color of the grid. */
-    gtk_style_context_get_color (context, GTK_STATE_FLAG_NORMAL, &fg_grid);
+    gtk_style_context_get_color (context, gtk_widget_get_state_flags (GTK_WIDGET (GsmApplication::get()->stack)), &fg_grid);
 
     /* Why not use the new features of the
      * GTK instead of cairo_rectangle ?! :) */
@@ -125,7 +228,7 @@ static void draw_background(LoadGraph *graph) {
     gtk_style_context_restore (context);
 
     cairo_set_line_width (cr, 1.0);
-    
+
     for (i = 0; i <= num_bars; ++i) {
         double y;
 
@@ -137,20 +240,16 @@ static void draw_background(LoadGraph *graph) {
             y = i * graph->graph_dely + graph->fontsize / 2.0;
 
         gdk_cairo_set_source_rgba (cr, &fg);
-        if (graph->type == LOAD_GRAPH_NET) {
-            // operation orders matters so it's 0 if i == num_bars
-            guint64 rate = graph->net.max - (i * graph->net.max / num_bars);
-            const std::string captionstr(procman::format_network_rate(rate));
-            caption = g_strdup(captionstr.c_str());
-        } else {
-            // operation orders matters so it's 0 if i == num_bars
-            caption = g_strdup_printf("%d %%", 100 - i * (100 / num_bars));
-        }
+        caption = graph->get_caption(i);
         pango_layout_set_alignment (layout, PANGO_ALIGN_LEFT);
         pango_layout_set_text (layout, caption, -1);
         pango_layout_get_extents (layout, NULL, &extents);
+        label_y_offset_modifier = i == 0 ? 0.5
+                                : i == num_bars
+                                    ? 1.0
+                                    : 0.85;
         cairo_move_to (cr, graph->draw_width - graph->indent - 23,
-                       y - 1.0 * extents.height / PANGO_SCALE / 2);
+                       y - label_y_offset_modifier * extents.height / PANGO_SCALE);
         pango_cairo_show_layout (cr, layout);
         g_free(caption);
 
@@ -164,9 +263,9 @@ static void draw_background(LoadGraph *graph) {
         cairo_line_to (cr, graph->draw_width - graph->rmargin + 0.5 + 4, i * graph->graph_dely + 0.5);
         cairo_stroke (cr);
     }
-    
 
-    const unsigned total_seconds = graph->speed * (LoadGraph::NUM_POINTS - 2) / 1000;
+
+    const unsigned total_seconds = graph->speed * (graph->num_points - 2) / 1000 * graph->frames_per_unit;
 
     for (unsigned int i = 0; i < 7; i++) {
         double x = (i) * (graph->draw_width - graph->rmargin - graph->indent) / 6;
@@ -180,17 +279,17 @@ static void draw_background(LoadGraph *graph) {
         cairo_move_to (cr, (ceil(x) + 0.5) + graph->indent, 0.5);
         cairo_line_to (cr, (ceil(x) + 0.5) + graph->indent, graph->real_draw_height + 4.5);
         cairo_stroke(cr);
-        unsigned seconds = total_seconds - i * total_seconds / 6;
-        const char* format;
-        if (i == 0)
-            format = dngettext(GETTEXT_PACKAGE, "%u second", "%u seconds", seconds);
-        else
-            format = "%u";
-        caption = g_strdup_printf(format, seconds);
+
+        caption = format_duration(total_seconds - i * total_seconds / 6);
+
         pango_layout_set_text (layout, caption, -1);
         pango_layout_get_extents (layout, NULL, &extents);
+        label_x_offset_modifier = i == 0 ? 0
+                                         : i == 6
+                                            ? 1.0
+                                            : 0.5;
         cairo_move_to (cr,
-                       (ceil(x) + 0.5 + graph->indent) - (1.0 * extents.width / PANGO_SCALE / 2),
+                       (ceil(x) + 0.5 + graph->indent) - label_x_offset_modifier * extents.width / PANGO_SCALE + 1.0,
                        graph->draw_height - 1.0 * extents.height / PANGO_SCALE);
         gdk_cairo_set_source_rgba (cr, &fg);
         pango_cairo_show_layout (cr, layout);
@@ -213,6 +312,15 @@ load_graph_queue_draw (LoadGraph *graph)
 void load_graph_update_data (LoadGraph *graph);
 static int load_graph_update (gpointer user_data); // predeclare load_graph_update so we can compile ;)
 
+static void
+load_graph_rescale (LoadGraph *graph) {
+    ///org/gnome/desktop/interface/text-scaling-factor
+    graph->fontsize = 8 * graph->font_settings->get_double ("text-scaling-factor");
+    graph->clear_background();
+
+    load_graph_queue_draw (graph);
+}
+
 static gboolean
 load_graph_configure (GtkWidget *widget,
                       GdkEventConfigure *event,
@@ -221,6 +329,8 @@ load_graph_configure (GtkWidget *widget,
     GtkAllocation allocation;
     LoadGraph * const graph = static_cast<LoadGraph*>(data_ptr);
 
+    load_graph_rescale (graph);
+
     gtk_widget_get_allocation (widget, &allocation);
     graph->draw_width = allocation.width - 2 * FRAME_WIDTH;
     graph->draw_height = allocation.height - 2 * FRAME_WIDTH;
@@ -253,6 +363,7 @@ load_graph_state_changed (GtkWidget *widget,
 {
     LoadGraph * const graph = static_cast<LoadGraph*>(data_ptr);
     force_refresh (graph);
+    graph->draw = gtk_widget_is_visible (widget);
     return TRUE;
 }
 
@@ -268,7 +379,7 @@ load_graph_draw (GtkWidget *widget,
     gdouble sample_width, x_offset;
 
     /* Number of pixels wide for one sample point */
-    sample_width = (double)(graph->draw_width - graph->rmargin - graph->indent) / (double)LoadGraph::NUM_POINTS;
+    sample_width = (double)(graph->draw_width - graph->rmargin - graph->indent) / (double)graph->num_points;
     /* Lines start at the right edge of the drawing,
      * a bit outside the clip rectangle. */
     x_offset = graph->draw_width - graph->rmargin + sample_width + 2;
@@ -292,34 +403,34 @@ load_graph_draw (GtkWidget *widget,
     cairo_clip(cr);
 
     bool drawStacked = graph->type == LOAD_GRAPH_CPU && GsmApplication::get()->config.draw_stacked;
-    bool drawSmooth = graph->type != LOAD_GRAPH_CPU || GsmApplication::get()->config.draw_smooth;
+    bool drawSmooth = GsmApplication::get()->config.draw_smooth;
     for (j = graph->n-1; j >= 0; j--) {
         gdk_cairo_set_source_rgba (cr, &(graph->colors [j]));
         // Start drawing on the right at the correct height.
-        cairo_move_to (cr, x_offset, (1.0f - graph->data[0][j]) * graph->real_draw_height + 3.5f);
+        cairo_move_to (cr, x_offset, (1.0f - graph->data[0][j]) * graph->real_draw_height + 3);
         // then draw the path of the line.
         // Loop starts at 1 because the curve accesses the 0th data point.
-        for (i = 1; i < LoadGraph::NUM_POINTS; ++i) {
+        for (i = 1; i < graph->num_points; ++i) {
             if (graph->data[i][j] == -1.0f)
                 continue;
             if (drawSmooth) {
                 cairo_curve_to (cr,
                               x_offset - ((i - 0.5f) * graph->graph_delx),
-                              (1.0 - graph->data[i-1][j]) * graph->real_draw_height + 3.5,
+                              (1.0 - graph->data[i-1][j]) * graph->real_draw_height + 3,
                               x_offset - ((i - 0.5f) * graph->graph_delx),
-                              (1.0 - graph->data[i][j]) * graph->real_draw_height + 3.5,
+                              (1.0 - graph->data[i][j]) * graph->real_draw_height + 3,
                               x_offset - (i * graph->graph_delx),
-                              (1.0 - graph->data[i][j]) * graph->real_draw_height + 3.5);
+                              (1.0 - graph->data[i][j]) * graph->real_draw_height + 3);
             } else {
                 cairo_line_to (cr, x_offset - (i * graph->graph_delx),
-                              (1.0 - graph->data[i][j]) * graph->real_draw_height + 3.5);
+                              (1.0 - graph->data[i][j]) * graph->real_draw_height + 3);
             }
 
         }
         if (drawStacked) {
             // Draw the remaining outline of the area:
             // Left bottom corner
-            cairo_rel_line_to (cr, 0, graph->real_draw_height + 3.5);
+            cairo_rel_line_to (cr, 0, graph->real_draw_height + 3);
             // Right bottom corner. It's drawn far outside the visible area
             // to avoid a weird bug where it's not filling the area it should completely.
             cairo_rel_line_to (cr, x_offset * 2, 0);
@@ -409,17 +520,17 @@ namespace
         char* total_text;
         char* text;
 
-        used_text = g_format_size_full(used, G_FORMAT_SIZE_IEC_UNITS);
-        cached_text = g_format_size_full(cached, G_FORMAT_SIZE_IEC_UNITS);
-        total_text = g_format_size_full(total, G_FORMAT_SIZE_IEC_UNITS);
+        used_text = format_byte_size(used, GsmApplication::get()->config.resources_memory_in_iec);
+        cached_text = format_byte_size(cached, GsmApplication::get()->config.resources_memory_in_iec);
+        total_text = format_byte_size(total, GsmApplication::get()->config.resources_memory_in_iec);
         if (total == 0) {
             text = g_strdup(_("not available"));
         } else {
-            // xgettext: 540MiB (53 %) of 1.0 GiB
+            // xgettext: "540MiB (53 %) of 1.0 GiB" or "540MB (53 %) of 1.0 GB"
             text = g_strdup_printf(_("%s (%.1f%%) of %s"), used_text, 100.0 * percent, total_text);
 
             if (cached != 0) {
-                // xgettext: Used cache string, e.g.: "Cache 2.4GiB"
+                // xgettext: Used cache string, e.g.: "Cache 2.4GiB" or "Cache 2.4GB"
                 cached_label = g_strdup_printf(_("Cache %s"), cached_text);
                 text = g_strdup_printf("%s\n%s", text, cached_label);
                 g_free (cached_label);
@@ -457,11 +568,11 @@ get_memory (LoadGraph *graph)
     set_memory_label_and_picker(GTK_LABEL(graph->labels.swap),
                                 GSM_COLOR_BUTTON(graph->swap_color_picker),
                                 swap.used, 0, swap.total, swappercent);
-    
+
     gtk_widget_set_sensitive (GTK_WIDGET (graph->swap_color_picker), swap.total > 0);
-    
-    graph->data[0][0] = mempercent;
-    graph->data[0][1] = swap.total>0 ? swappercent : -1.0;
+
+    graph->data[0][0] = graph->translate_to_log_partial_if_needed(mempercent);
+    graph->data[0][1] = swap.total>0 ? graph->translate_to_log_partial_if_needed(swappercent) : -1.0;
 }
 
 /* Nice Numbers for Graph Labels after Paul Heckbert
@@ -471,12 +582,12 @@ get_memory (LoadGraph *graph)
 static double
 nicenum (double x, int round)
 {
-    int expv;				/* exponent of x */
-    double f;				/* fractional part of x */
-    double nf;				/* nice, rounded fraction */
+    int expv;                /* exponent of x */
+    double f;                /* fractional part of x */
+    double nf;                /* nice, rounded fraction */
 
     expv = floor(log10(x));
-    f = x/pow(10.0, expv);		/* between 1 and 10 */
+    f = x/pow(10.0, expv);        /* between 1 and 10 */
     if (round) {
         if (f < 1.5)
             nf = 1.0;
@@ -506,8 +617,11 @@ net_scale (LoadGraph *graph, guint64 din, guint64 dout)
     graph->data[0][1] = 1.0f * dout / graph->net.max;
 
     guint64 dmax = std::max(din, dout);
-    graph->net.values[graph->net.cur] = dmax;
-    graph->net.cur = (graph->net.cur + 1) % LoadGraph::NUM_POINTS;
+    if (graph->latest == 0) {
+        graph->net.values[graph->num_points - 1] = dmax;
+    } else {
+        graph->net.values[graph->latest - 1] = dmax;
+    }
 
     guint64 new_max;
     // both way, new_max is the greatest value
@@ -515,7 +629,7 @@ net_scale (LoadGraph *graph, guint64 din, guint64 dout)
         new_max = dmax;
     else
         new_max = *std::max_element(&graph->net.values[0],
-                                    &graph->net.values[LoadGraph::NUM_POINTS]);
+                                    &graph->net.values[graph->num_points - 1]);
 
     //
     // Round network maximum
@@ -584,7 +698,7 @@ net_scale (LoadGraph *graph, guint64 din, guint64 dout)
 
     const double scale = 1.0f * graph->net.max / new_max;
 
-    for (size_t i = 0; i < LoadGraph::NUM_POINTS; i++) {
+    for (size_t i = 0; i < graph->num_points; i++) {
         if (graph->data[i][0] >= 0.0f) {
             graph->data[i][0] *= scale;
             graph->data[i][1] *= scale;
@@ -679,8 +793,11 @@ load_graph_update_data (LoadGraph *graph)
 {
     // Rotate data one element down.
     std::rotate(&graph->data[0],
-                &graph->data[LoadGraph::NUM_POINTS - 1],
-                &graph->data[LoadGraph::NUM_POINTS]);
+                &graph->data[graph->num_points - 2],
+                &graph->data[graph->num_points - 1]);
+
+    // Update rotation counter.
+    graph->latest = (graph->latest + 1) % graph->num_points;
 
     // Replace the 0th element
     switch (graph->type) {
@@ -747,11 +864,13 @@ load_graph_destroy (GtkWidget *widget, gpointer data_ptr)
 
 LoadGraph::LoadGraph(guint type)
     : fontsize(8.0),
-      rmargin(7 * fontsize),
-      indent(24.0),
+      rmargin(6 * fontsize),
+      indent(18.0),
       n(0),
       type(type),
       speed(0),
+      num_points(0),
+      latest(0),
       draw_width(0),
       draw_height(0),
       render_counter(0),
@@ -762,6 +881,7 @@ LoadGraph::LoadGraph(guint type)
       graph_buffer_offset(0),
       colors(),
       data_block(),
+      data(),
       main_widget(NULL),
       disp(NULL),
       background(NULL),
@@ -770,60 +890,61 @@ LoadGraph::LoadGraph(guint type)
       labels(),
       mem_color_picker(NULL),
       swap_color_picker(NULL),
+      font_settings(Gio::Settings::create (FONT_SETTINGS_SCHEMA)),
       cpu(),
       net()
 {
     LoadGraph * const graph = this;
-
+    font_settings->signal_changed(FONT_SETTING_SCALING).connect([this](const Glib::ustring&) { load_graph_rescale (this); } );
     // FIXME:
     // on configure, graph->frames_per_unit = graph->draw_width/(LoadGraph::NUM_POINTS);
     // knock FRAMES down to 5 until cairo gets faster
 
     switch (type) {
         case LOAD_GRAPH_CPU:
-            memset(&cpu, 0, sizeof cpu);
+            cpu = CPU {};
             n = GsmApplication::get()->config.num_cpus;
 
             for(guint i = 0; i < G_N_ELEMENTS(labels.cpu); ++i)
-                labels.cpu[i] = GTK_LABEL (gtk_label_new(NULL));
+                labels.cpu[i] = make_tnum_label ();
 
             break;
 
         case LOAD_GRAPH_MEM:
             n = 2;
-            labels.memory = GTK_LABEL (gtk_label_new(NULL));
+            labels.memory = make_tnum_label ();
             gtk_widget_set_valign (GTK_WIDGET (labels.memory), GTK_ALIGN_CENTER);
             gtk_widget_set_halign (GTK_WIDGET (labels.memory), GTK_ALIGN_START);
             gtk_widget_show (GTK_WIDGET (labels.memory));
-            labels.swap = GTK_LABEL (gtk_label_new(NULL));
+            labels.swap = make_tnum_label ();
             gtk_widget_set_valign (GTK_WIDGET (labels.swap), GTK_ALIGN_CENTER);
             gtk_widget_set_halign (GTK_WIDGET (labels.swap), GTK_ALIGN_START);
             gtk_widget_show (GTK_WIDGET (labels.swap));
             break;
 
         case LOAD_GRAPH_NET:
-            memset(&net, 0, sizeof net);
+            net = NET {};
             n = 2;
             net.max = 1;
-            labels.net_in = GTK_LABEL (gtk_label_new(NULL));
+            labels.net_in = make_tnum_label ();
             gtk_label_set_width_chars(labels.net_in, 10);
             gtk_widget_set_valign (GTK_WIDGET (labels.net_in), GTK_ALIGN_CENTER);
             gtk_widget_set_halign (GTK_WIDGET (labels.net_in), GTK_ALIGN_END);
             gtk_widget_show (GTK_WIDGET (labels.net_in));
 
-            labels.net_in_total = GTK_LABEL (gtk_label_new(NULL));
+            labels.net_in_total = make_tnum_label ();
             gtk_widget_set_valign (GTK_WIDGET (labels.net_in_total), GTK_ALIGN_CENTER);
             gtk_widget_set_halign (GTK_WIDGET (labels.net_in_total), GTK_ALIGN_END);
             gtk_label_set_width_chars(labels.net_in_total, 10);
             gtk_widget_show (GTK_WIDGET (labels.net_in_total));
 
-            labels.net_out = GTK_LABEL (gtk_label_new(NULL));
+            labels.net_out = make_tnum_label ();
             gtk_widget_set_valign (GTK_WIDGET (labels.net_out), GTK_ALIGN_CENTER);
             gtk_widget_set_halign (GTK_WIDGET (labels.net_out), GTK_ALIGN_END);
             gtk_label_set_width_chars(labels.net_out, 10);
             gtk_widget_show (GTK_WIDGET (labels.net_out));
 
-            labels.net_out_total = GTK_LABEL (gtk_label_new(NULL));
+            labels.net_out_total = make_tnum_label ();
             gtk_widget_set_valign (GTK_WIDGET (labels.net_out_total), GTK_ALIGN_CENTER);
             gtk_widget_set_halign (GTK_WIDGET (labels.net_out), GTK_ALIGN_END);
             gtk_label_set_width_chars(labels.net_out_total, 10);
@@ -832,7 +953,9 @@ LoadGraph::LoadGraph(guint type)
             break;
     }
 
-    speed  = GsmApplication::get()->config.graph_update_interval;
+    speed = GsmApplication::get()->config.graph_update_interval;
+
+    num_points = GsmApplication::get()->config.graph_data_points + 2;
 
     colors.resize(n);
 
@@ -850,6 +973,7 @@ LoadGraph::LoadGraph(guint type)
                                                          GSMCP_TYPE_PIE);
             break;
         case LOAD_GRAPH_NET:
+            net.values = std::vector<unsigned>(num_points);
             colors[0] = GsmApplication::get()->config.net_in_color;
             colors[1] = GsmApplication::get()->config.net_out_color;
             break;
@@ -880,11 +1004,11 @@ LoadGraph::LoadGraph(guint type)
 
     gtk_box_pack_start (main_widget, GTK_WIDGET (disp), TRUE, TRUE, 0);
 
-
+    data = std::vector<double*>(num_points);
     /* Allocate data in a contiguous block */
-    data_block = std::vector<double>(n * LoadGraph::NUM_POINTS, -1.0);
+    data_block = std::vector<double>(n * num_points, -1.0);
 
-    for (guint i = 0; i < LoadGraph::NUM_POINTS; ++i)
+    for (guint i = 0; i < num_points; ++i)
         data[i] = &data_block[0] + i * n;
 
     gtk_widget_show_all (GTK_WIDGET (main_widget));
@@ -899,7 +1023,7 @@ load_graph_start (LoadGraph *graph)
         load_graph_update_data(graph);
         load_graph_update(graph);
 
-        graph->timer_index = g_timeout_add (graph->speed / graph->frames_per_unit,
+        graph->timer_index = g_timeout_add (graph->speed,
                                             load_graph_update,
                                             graph);
     }
@@ -925,7 +1049,7 @@ load_graph_change_speed (LoadGraph *graph,
 
     if (graph->timer_index) {
         g_source_remove (graph->timer_index);
-        graph->timer_index = g_timeout_add (graph->speed / graph->frames_per_unit,
+        graph->timer_index = g_timeout_add (graph->speed,
                                             load_graph_update,
                                             graph);
     }
@@ -933,6 +1057,47 @@ load_graph_change_speed (LoadGraph *graph,
     graph->clear_background();
 }
 
+void
+load_graph_change_num_points(LoadGraph *graph,
+                             guint new_num_points)
+{
+    //Don't do anything if the value didn't change.
+    if (graph->num_points == new_num_points)
+        return;
+
+    // Sort the values in the data_block vector in the order they were accessed in by the pointers in data.
+    std::rotate(&graph->data_block[0],
+                &graph->data_block[(graph->num_points - graph->latest) * graph->n],
+                &graph->data_block[graph->num_points * graph->n]);
+
+    // Reset rotation counter.
+    graph->latest = 0;
+
+    // Resize the vectors to the new amount of data points.
+    graph->data.resize(new_num_points);
+    graph->data_block.resize(graph->n * new_num_points);
+    if (graph->type == LOAD_GRAPH_NET) {
+        graph->net.values.resize(new_num_points);
+    }
+
+    // Fill the new values with -1 instead of 0 if the vectors got bigger.
+    if (new_num_points > graph->num_points) {
+        std::fill(&graph->data_block[graph->n * graph->num_points],
+                  &graph->data_block[graph->n * new_num_points], -1.0);
+    }
+
+    // Replace the pointers in data, to match the new data_block values.
+    for (guint i = 0; i < new_num_points; ++i) {
+        graph->data[i] = &graph->data_block[0] + i * graph->n;
+    }
+
+    // Set the actual number of data points to be used by the graph.
+    graph->num_points = new_num_points;
+
+    // Force the scale to be redrawn.
+    graph->clear_background();
+}
+
 
 LoadGraphLabels*
 load_graph_get_labels (LoadGraph *graph)
